local DG = DoomGuy
local save_path = "client/doomguy"
local save_file = "client"
local save_extension = "cfg"
local function Save()
	if not consoleplayer then return end

	local file_target = save_path .. "/" .. save_file .. "." .. save_extension
	local file_config = io.openlocal(file_target, "w+")
	file_config:write("config_version: " .. DG.config_version, "\n", "\n")
	for index, cvar in pairs(doomclient) do
		file_config:write(cvar.name .. ": " .. cvar.string, "\n")
	end
	file_config:close()
	print("DoomGuy: Client config file at " .. file_target .. " saved")
end

local function Load()
	if not consoleplayer then return end

	local file_target = save_path .. "/" .. save_file .. "." .. save_extension
	local file_config = io.openlocal(file_target, "r")
	if not file_config then -- file check
		print("DoomGuy: Client config file at " .. file_target .. " don't exists, created the file")
		Save()
		return
	end
	
	local total_commands = {}
	local line_index = 0
	for line in file_config:lines() do -- find cvars
		local reader = DG.SplitString(": ", line)
		if not reader[2] then continue end
		
		local command, value = reader[1], reader[2]
		if not line_index then -- version check
			if (DG.config_version ~= tonumber(value)) then
				print("DoomGuy: Client config file at " .. file_target .. " was saved in a different config version, still loading the file")
			end
			line_index = $ + 1
			continue
		end
		
		local find_cvar = CV_FindVar(command)
		if not find_cvar then continue end
		if (find_cvar.string == value) then continue end
	
		table.insert(total_commands, {command, value})
		line_index = $ + 1
	end
	
	if not line_index then -- invalid file check
		io.close(file_config)
		print("DoomGuy: Client config file at " .. file_target .. " is empty or corrupted, re-created the file")
		Save()
		return
	end
	io.close(file_config)
	return total_commands
end

local load_cvars = {}
local loaded_cvars, loaded_file = false, false
addHook("PreThinkFrame", function()
	if isdedicatedserver then return end
	if loaded_cvars then return end
	if (gamestate ~= GS_LEVEL) then return end
	if not consoleplayer then return end
	
	if not loaded_file then
		load_cvars = Load() or {}
		loaded_file = true
		return
	end
	
	if (#load_cvars <= 0) then
		local file_target = save_path .. "/" .. save_file .. "." .. save_extension
		print("DoomGuy: Client config file at " .. file_target .. " loaded")
		loaded_cvars = true
		return
	end
	
	local cvar = table.remove(load_cvars)
	local find_cvar = CV_FindVar(cvar[1])
	if not find_cvar then return end
	if (find_cvar.string == cvar[2]) then return end
	COM_BufInsertText(consoleplayer, find_cvar.name .. ' "' .. cvar[2] .. '"')
end)

addHook("GameQuit", function(quitting)
	if isdedicatedserver then return end
	Save()
	load_cvars = {}
	loaded_cvars, loaded_file = false, false
end)