local DG = DoomGuy
DG.Weapons = {[1] = {}, [2] = {}, [3] = {}, [4] = {}, [5] = {},[6] = {}, [7] = {} }

function DG.AddWeapon(slot, weapon)
	if (slot < 0) or (slot > 7) then
		print("DG.AddWeapon: Invalid slot specified, should be between 1 and 7")
		return
	end
	
	for name, struct in pairs(weapon) do
		DG.Weapons[slot][name] = struct
		print("DG.AddWeapon: Added weapon '" .. name .. "' at slot " .. slot)
	end
end

function DG.GetWeapon(slot, weapon)
	if not DG.Weapons[slot] then return end
	return DG.Weapons[slot][weapon]
end

function DG.GetCurrentPlayerWeapon(player)
	if not player.doom then return end
	if not DG.Weapons[player.doom.weapon_slot] then return end
	return DG.GetWeapon(player.doom.weapon_slot, player.doom.weapon)
end

function DG.Fire(source, mobj, speed, range, damage, damagetype, hangle, vangle, hspread, vspread, xoff, yoff, zoff, flags, flags2)
	local bullet = P_SpawnMobjFromMobj(source, xoff, yoff, zoff, mobj)
	bullet.target = source
	bullet.damage, bullet.damagetype = damage, damagetype
	bullet.flags, bullet.flags2 = $1 | flags, $2 | flags2
	bullet.threshold = range
	
	speed = FixedMul(speed, bullet.scale)
	hangle, vangle = $1 + (P_SignedRandom() * hspread), $2 + (P_SignedRandom() * vspread)
	
	bullet.momx, bullet.momy, bullet.momz = FixedMul(FixedMul(speed, cos(hangle)), cos(vangle)), FixedMul(FixedMul(speed, sin(hangle)), cos(vangle)), FixedMul(speed, sin(vangle))
	bullet.hangle, bullet.vangle = hangle, vangle
	return bullet
end

function DG.FirePlayer(player, mobj, range, damage, damagetype, hangle, vangle, hspread, vspread, flags, flags2)
	return DG.Fire(player.realmo, mobj, mobjinfo[mobj].speed, range, damage, damagetype, hangle, vangle, hspread, vspread, 0, 0, (32 * FU), flags, flags2)
end

function DG.PlayerHasWeapon(player, id)
	if doomcheats.skip_weapon_conditions.value then return true end
	if not player.doom.weapons[id] then return false end
	return true
end

function DG.PlayerHasAmmoForWeapon(player, weapon)
	if (weapon.ammo_usage == nil) then return true end
	if doomcheats.infinite_ammo.value then return true end
	if doomcheats.skip_weapon_conditions.value then return true end
	if not player.doom.ammo[weapon.ammo_type] then return false end
	if (player.doom.ammo[weapon.ammo_type] < weapon.ammo_usage) then return false end
	return true
end

function DG.ConsumePlayerAmmo(player, weapon)
	if doomcheats.skip_weapon_conditions.value then return true end
	if not DG.PlayerHasAmmoForWeapon(player, player.doom.weapon, weapon) then return false end
	player.doom.ammo[weapon.ammo_type] = max($ - weapon.ammo_usage, 0)
	return true
end

function DG.PlayerCanUseWeapon(player, id, weapon)
	if not DG.PlayerHasWeapon(player, id, weapon) then return false end
	if not DG.PlayerHasAmmoForWeapon(player, weapon) then return false end
	return true
end

function DG.PlayerRefire(player)
	local wpn = DG.GetCurrentPlayerWeapon(player)
	if not wpn then return end
	if not DG.PlayerCanUseWeapon(player, player.doom.weapon, wpn) then return end
	
	if (player.doom.weapon ~= player.doom.pending_weapon) then
		player.doom.state = WEAPONSTATE_LOWER
		player.doom.state_index = 1
		player.doom.tick = wpn.states[player.doom.state][player.doom.state_index].tics
		DG.UpdatePlayerWeaponSprites(player, wpn)
		DG.ExecutePlayerWeaponState(player, wpn)
		return
	end
	
	if (player.cmd.buttons & BT_ATTACK) then
		player.doom.state = WEAPONSTATE_ATTACK
		player.doom.state_index = 1
		player.doom.tick = wpn.states[player.doom.state][player.doom.state_index].tics
		DG.UpdatePlayerWeaponSprites(player, wpn)
		DG.ExecutePlayerWeaponState(player, wpn)
		player.doom.refire = $ + 1
	else
		player.doom.refire = 0
	end
end

function DG.GivePlayerAmmo(player, ammo_type, amount)
	if not DG.Ammos[ammo_type] then return end
	
	if (player.doom.ammo[ammo_type] >= player.doom.ammo_max[ammo_type]) then return end
	player.doom.ammo[ammo_type] = min($ + amount, player.doom.ammo_max[ammo_type])
	return true
end

function DG.GivePlayerWeapon(player, slot, weapon)
	if not DG.Weapons[slot] or not DG.Weapons[slot][weapon] then return end
	if player.doom.weapons[weapon] then return end
	player.doom.weapons[weapon] = true
	
	if (DG.Weapons[slot][weapon].ammo_usage ~= nil) then
		local amount = (gametyperules & GTR_CAMPAIGN) and 2 or 5
		local ammo_type = DG.Weapons[slot][weapon].ammo_type
		local ammo_give = DG.Ammos[ammo_type].give_amount
		DG.GivePlayerAmmo(player, ammo_type, ammo_give * amount)
	end
	
	if DG.PlayerCanUseWeapon(player, weapon, DG.Weapons[slot][weapon]) and not doomcheats.skip_weapon_conditions.value then
		player.doom.pending_weapon = weapon
		player.doom.pending_weapon_slot = slot
	end
	return true
end

function DG.DropBlood(x, y, z, damage)
	local pos_x, pos_y, pos_z = x, y, z
	pos_z = $ + ((P_RandomByte() - P_RandomByte()) << 10)
	
	local mobj = P_SpawnMobj(pos_x, pos_y, pos_z, MT_DBLOOD)
	mobj.momz = FU * 2
	mobj.tics = $ - P_RandomByte() & 3
	if (mobj.tics < 1) then
		mobj.tics = 1
	end
	
	if (damage <= 12) and (damage >= 9) then
		mobj.state = S_BLOOD_EFFECT2
	elseif (damage < 9) then
		mobj.state = S_BLOOD_EFFECT3
	end
	return mobj
end