local DG = DoomGuy

-- Run once to cache the whole font
local cached = false
local function create_font(v, fntname, fntpref, mono, space, height)
	rawset(_G, fntname, {monospace = mono, space = space, height = height})
	for i = 0, 255 do
		local build = string.format(fntpref .. "%03d", i)
		if v.patchExists(build)
			_G[fntname][i] = v.cachePatch(build)
		end
	end
end

hud.add(function(v, p, c)
	if (FONT_DOOMNORMAL == nil) or (FONT_DOOMNORMAL[37] == nil) or not FONT_DOOMNORMAL[37].valid
		cached = false
	end
	if cached return end

	create_font(v, "FONT_DOOMNORMAL", "DMFT1", 8, 4, 8)
	create_font(v, "FONT_DOOMTHIN", "DMFT2", 6, 4, 8)
	create_font(v, "FONT_DOOMCONSOLE", "DMFTA", 6, 4, 8)
	create_font(v, "FONT_PRBOOM", "DIG0", 6, 4, 8)
	cached = true
end, "game")

-- Font Styles
local styles = {
	"FS_MONOSPACE", -- Fixed width
	"FS_FIXED", -- Enables scaling and places things in the fixed scale, epic!
	"FS_FIXEDSTRECH"
}

-- Set styles
for i = 1, #styles
	rawset(_G, styles[i], 2 ^ (i - 1))
end

--
hud.add(function(v, p, c)
    if drawCustomString return end
    
    rawset(_G, "drawCustomString", function(x, y, str, font, flags, extra) -- EXTRA: {color = , style = , align = , scale = }
		local style, align, color, scale, yscale
		if (extra ~= nil)
			color = extra.color
			style = extra.style
			align = extra.align
			scale = extra.scale
			yscale = extra.yscale
		end
		
		if (str == true) then str = "true" end
		if (str == false) then str = "false" end
		if (type(str) == "number") then str = tostring(str) end
		if (type(str) ~= "string") then str = type(str) end
		
		if (style == nil) then style = 0 end
		if (color == nil) then color = 0 end
		if (scale == nil) then scale = FRACUNIT end
		if (yscale == nil) then yscale = FRACUNIT end
		if (type(color) ~= "table") then color = {color} end
		
        local dt = {}
        local offset = 0
		local mono = font.monospace
        for i = 1, #str do
            local by = string.byte(str:sub(i, i))
			local res = 0
            if (font[by] ~= nil) then
                dt[#dt + 1] = {font[by], offset}
				res = font[by].width
            else
				res = mono
            end
			
			if (style & FS_MONOSPACE) then
				res = mono
			end
			
			if (style & FS_FIXED) then
				res = FixedMul(res * FRACUNIT, scale)
			end
			offset = $ + res
        end

        local eo = 0
        if (align == "center") then eo = (offset / 2) end
        if (align == "right") then eo = offset end
		
		local colormaps = {}
		for i = 1, #color do
			colormaps[i] = v.getColormap(0, color[i])
		end
		
        for i = 1, #dt do
            local ix = dt[i]
			if (style & FS_FIXED) then
				if (style & FS_FIXEDSTRECH) then
					v.drawStretched(x + ix[2] - eo, y, scale, yscale, ix[1], flags, colormaps[1])
				else
					v.drawScaled(x + ix[2] - eo, y, scale, ix[1], flags, colormaps[1])
				end
			else
				v.draw(x + ix[2] - eo, y, ix[1], flags, colormaps[1])
			end
        end
    end)
end)