local DG = DoomGuy
addHook("MobjDamage", function(target, inflictor, source, damage, damagetype)
	if not (target.flags & (MF_ENEMY|MF_BOSS)) then return end
	
	local attacker = inflictor or source
	if not attacker or not attacker.valid then return end
	
	if attacker.damage then
		local actual_damage = (target.flags & MF_BOSS) and (attacker.damage / 4) or (attacker.damage / 2)
		target.health = max($ - actual_damage, 0)
	end
	
	if attacker.bfgtracer then
		P_SpawnMobjFromMobj(target, 0, 0, target.height >> 2, MT_DBFGHIT)
	end
	
	if attacker.meleeattack and source and source.valid then
		source.angle = R_PointToAngle2(target.x, target.y, source.x, source.y) - ANGLE_180
		
		local sounds = {sfx_sawup, sfx_sawidl, sfx_sawful, sfx_sawhit, sfx_fistpu}
		for index = 1, #sounds, 1 do
			S_StopSoundByID(source, sounds[index])
		end
		S_StartSound(source, ({sfx_fistpu, sfx_sawhit})[attacker.meleeattack], nil)
	end
end)

local ammoDrops = {MT_AMMO_CLIP, MT_AMMO_SHELL, MT_AMMO_ROCKET, MT_AMMO_CELL}
local weaponDrops = {MT_WEAPON_SHOTGUN, MT_WEAPON_SUPERSHOTGUN, MT_WEAPON_CHAINGUN, MT_WEAPON_ROCKETLAUNCHER, MT_WEAPON_PLASMARIFLE, MT_WEAPON_BFG9000, MT_WEAPON_CHAINSAW}
local bigAmmoDrops = {MT_AMMO_CLIP_BOX, MT_AMMO_SHELL_BOX, MT_AMMO_ROCKET_BOX, MT_AMMO_CELL_PACK}
local powerupDrops = {MT_POWERUP_BACKPACK, MT_POWERUP_HAZARDSUIT, MT_POWERUP_BERSERK}
local itemsDrops = {MT_ITEM_HEALTH, MT_ITEM_ARMOR}
local specialItemsDrops = {MT_ITEM_STIMPACK, MT_ITEM_HEALTHPACK, MT_ITEM_SECURITY_ARMOR, MT_ITEM_COMBAT_ARMOR, MT_ITEM_SOUL, MT_ITEM_MEGA, MT_ITEM_INVULNERABILITY}
local function SpawnDropMobj(source, loot)
	local drop = P_SpawnMobjFromMobj(source, 0, 0, 0, loot)
	P_Thrust(drop, FixedAngle(P_RandomRange(0, 360) * FU), 2 * FU)
	P_SetObjectMomZ(drop, 5 * FU, false)
	
	drop.fuse = 30 * TICRATE
	drop.flags2 = MF2_DONTRESPAWN
end

local function ThereIsANearbyDoomGuy(target, source)
	if source and source.valid and source.player and source.player.valid then
		if DG.PlayerIsDoomGuy(source.player) and source.player.doom then
			return true
		end
	end

	local distance = 1024 * FU
	for player in players.iterate do
		if not DG.PlayerIsDoomGuy(player) then continue end
		if not player.doom then continue end
		local calculate = abs(R_PointToDist2(target.x, target.y, player.realmo.x, player.realmo.y))
		if (calculate >= distance) then continue end
		return true
	end
	return false
end

addHook("MobjDeath", function(target, inflictor, source, damagetype)
	if not (target.flags & (MF_ENEMY|MF_BOSS)) then return end
	if (target.eflags & (MFE_DOOMDONTDROP|MFE_DOOMENEMY)) then return end
	if not ThereIsANearbyDoomGuy(target, source) then return end
	
	local extra = (target.flags & MF_BOSS) and 2 or 1
	local amount = P_RandomRange(1, 2) * extra
	for i = 1, amount, 1 do
		SpawnDropMobj(target, ammoDrops[P_RandomRange(1, #ammoDrops)]) -- ammo
		if P_RandomChance(FU / 4) then -- weapons
			SpawnDropMobj(target, weaponDrops[P_RandomRange(1, #weaponDrops)])
		end
		
		if P_RandomChance(FU / 6) then -- big ammo
			SpawnDropMobj(target, bigAmmoDrops[P_RandomRange(1, #bigAmmoDrops)])
		end
		
		if P_RandomChance(FU / 8) then -- items
			SpawnDropMobj(target, itemsDrops[P_RandomRange(1, #itemsDrops)])
		end
		
		if P_RandomChance(FU / 10) then -- powerups
			SpawnDropMobj(target, powerupDrops[P_RandomRange(1, #powerupDrops)])
		end
		
		if P_RandomChance(FU / 12) then -- special items
			SpawnDropMobj(target, specialItemsDrops[P_RandomRange(1, #specialItemsDrops)])
		end
	end
end)