local DG = DoomGuy
local killDamage = {
	[DMG_INSTAKILL] = true,
	[DMG_DROWNED] = true,
	[DMG_SPACEDROWN] = true,
	[DMG_DEATHPIT] = true,
	[DMG_CRUSHED] = true,
	[DMG_SPECTATOR] = true
}

local damageValues = {
	[DMG_WATER] = 2 * FU,
	[DMG_FIRE] = 10 * FU,
	[DMG_ELECTRIC] = 5 * FU,
	[DMG_SPIKE] = 10 * FU,
	[DMG_NUKE] = 40 * FU,
}

local damageBypassArmor = {
	[DAMAGETYPE_DROWING] = true,
	[DAMAGETYPE_SPACEDROWING] = true
}

local inflictorDamage = {
	[MT_SPINFIRE] = 10 * FU,
	[MT_THROWNBOUNCE] = 15 * FU,
	[MT_THROWNINFINITY] = 5 * FU,
	[MT_THROWNAUTOMATIC] = 2 * FU,
	[MT_THROWNSCATTER] = 15 * FU,
	[MT_THROWNEXPLOSION] = 60 * FU,
	[MT_THROWNGRENADE] = 40 * FU,
	[MT_REDRING] = 5 * FU
}
addHook("ShouldDamage", function(target, inflictor, source, damage, damagetype)
	if not target.player or not target.player.valid then return end
	
	local player = target.player
	if not DG.PlayerIsDoomGuy(player) then return end
	if not player.doom then return end
	if (player.powers[pw_carry] == CR_NIGHTSMODE) then return end
	
	if player.powers[pw_flashing] return false end
	if player.powers[pw_invulnerability] return false end
	if player.powers[pw_super] return false end
	if (player.pflags & PF_GODMODE) return false end
	if player.doom.invulnerability_tick return false end

	for specialnum = 1, 4, 1 do
		if not P_PlayerTouchingSectorSpecial(player, 1, specialnum) then continue end
		if (leveltime & 31) then return false end
		if player.doom.powers[POWERS_RADIATION_SUIT] or ((specialnum == 3) and (P_RandomByte() < 5)) then return false end
		return true
	end
end, MT_PLAYER)

addHook("MobjDamage", function(target, inflictor, source, damage, damagetype)
	if not target.player or not target.player.valid then return end
	
	local player = target.player
	if not DG.PlayerIsDoomGuy(player) then return end
	if not player.doom then return end
	if (player.powers[pw_carry] == CR_NIGHTSMODE) then return end
	
	if player.powers[pw_flashing] return true end
	if player.powers[pw_invulnerability] return true end
	if player.powers[pw_super] return true end
	if (player.pflags & PF_GODMODE) return true end
	if player.doom.invulnerability_tick return true end
	
	local isInstakill = killDamage[damagetype] or false
	local fromPlayer = false -- no invincibility frames on getting damaged by a player
	local currentDamage = damage * FU
	if source and source.valid then
		if source.player and source.player.valid then
			local source_player = source.player
			if DG.PlayerIsDoomGuy(source_player) and source_player.doom then -- doomguy player
				if inflictor and inflictor.valid then
					currentDamage = inflictor.damage and (inflictor.damage * FU) or FU
					
					/*if (damagetype == DAMAGETYPE_EXPLOSION) then -- damn hack, SRB2's damage system is painful with radius based explosions
						local radius = inflictor.health
						local rocket_distance = R_PointToDist2(target.x, target.y, inflictor.x + inflictor.dmomx, inflictor.y + inflictor.dmomy)
						local taken_damage = radius - rocket_distance
						currentDamage = (taken_damage > 0) and taken_damage or 0
					end*/
					
					if inflictor.bfgtracer then
						P_SpawnMobjFromMobj(target, 0, 0, target.height >> 2, MT_DBFGHIT)
					end
					
					if inflictor.meleeattack then
						source.angle = R_PointToAngle2(target.x, target.y, source.x, source.y) - ANGLE_180
						S_StartSound(source, ({sfx_fistpu, sfx_sawhit})[inflictor.meleeattack], nil)
					end
				end
			else -- normal srb2 player
				local source_player = source.player
				if (source == inflictor) then
					if ((source_player.powers[pw_shield] & SH_NOSTACK) == SH_ELEMENTAL) and (source_player.pflags & PF_SHIELDABILITY) then
						currentDamage = 20 * FU
					elseif source_player.powers[pw_super] then
						currentDamage = 10 * FU
					else
						currentDamage = 5 * FU
					end
				elseif inflictor and inflictor.valid then
					currentDamage = inflictorDamage[inflictor.type] or 5 * FU
					if (inflictor.flags2 & MF2_RAILRING) then
						currentDamage = 50 * FU
					end
				end
			end
			fromPlayer = true
			isInstakill = false
		else
			if (source.flags & MF_ENEMY) then
				currentDamage = P_RandomRange(2, 10) * FU
			end
			
			if (source.flags & MF_BOSS) then
				currentDamage = P_RandomRange(10, 20) * FU
			end
		end
	elseif inflictor and inflictor.valid then
		if inflictor.target and inflictor.target then
			if (inflictor.target.flags & MF_ENEMY) then
				currentDamage = P_RandomRange(2, 10) * FU
			end
			
			if (inflictor.target.flags & MF_BOSS) then
				currentDamage = P_RandomRange(10, 20) * FU
			end
		end
	end
	
	if (damagetype == DAMAGETYPE_EXPLOSION) then
		currentDamage = $ + abs(damage)
	end
	
	local extra_damage = damageValues[damagetype] or 0
	currentDamage = $ + extra_damage
	
	if isInstakill then
		currentDamage = INT32_MAX
	end
	if (currentDamage <= 0) then return true end
	
	if (player.doom.armor > 0) and not damageBypassArmor[damagetype] then
		local base_efficiency = max(0, min(player.doom.armor_efficiency, 100 * FU))
		local actual_efficiency = FixedDiv(base_efficiency, 100 * FU)
		local absorbed_damage = FixedMul(currentDamage, actual_efficiency)
		if (player.doom.armor <= absorbed_damage) then
			absorbed_damage = player.doom.armor
			player.doom.armor_efficiency = 0
		end
		
		player.doom.armor = $ - absorbed_damage
		currentDamage = $ - absorbed_damage
	end
	
	player.tossdelay = TR
	P_PlayerFlagBurst(player, true)
	P_PlayerEmeraldBurst(player, true)
	if not damageBypassArmor[damagetype] then
		DG.DropBlood(player.realmo.x, player.realmo.y, player.realmo.z + FixedMul(player.realmo.height / 2, player.realmo.scale), currentDamage)
	end
	
	player.doom.health = $ - max(currentDamage, 0)
	if (player.doom.health <= 0) then
		P_KillMobj(target, inflictor, source, damagetype)
		return true
	end

	if not fromPlayer then
		player.doom.invulnerability_tick = TR / 2
	end

	player.realmo.state = S_PLAY_PAIN
	player.panim = PA_PAIN
	
	player.doom.attacker = inflictor and inflictor.target or source
	player.doom.attacker_time = TR
	player.doom.hurted_flash = min(FixedInt(currentDamage), 100)
	S_StartSound(target, sfx_plpain, nil)
	return true
end, MT_PLAYER)

local function SpawnDropMobj(source, loot)
	local drop = P_SpawnMobjFromMobj(source, 0, 0, 0, loot)
	P_Thrust(drop, FixedAngle(P_RandomRange(0, 360) * FU), 2 * FU)
	P_SetObjectMomZ(drop, 5 * FU, false)
	
	drop.fuse = 30 * TR
	drop.flags2 = MF2_DONTRESPAWN
end

local dropWeapons = {
	["chainsaw"] = MT_WEAPON_CHAINSAW,
	["pistol"] = MT_WEAPON_PISTOL,
	["shotgun"] = MT_WEAPON_SHOTGUN,
	["supershotgun"] = MT_WEAPON_SUPERSHOTGUN,
	["chaingun"] = MT_WEAPON_CHAINGUN,
	["rocketlauncher"] = MT_WEAPON_ROCKETLAUNCHER,
	["plasmarifle"] = MT_WEAPON_PLASMARIFLE,
	["bfg9000"] = MT_WEAPON_BFG9000
}
addHook("MobjDeath", function(target, inflictor, source, damage, damagetype)
	if not target.player or not target.player.valid then return end
	
	local player = target.player
	if not DG.PlayerIsDoomGuy(player) then return end
	if not player.doom then return end
	player.doom.dead = true
	
	if (netgame or multiplayer) and doomgameplay.dropweapons_on_death.value and not doomgameplay.keepinv_on_death.value then
		for weapon, has in pairs(player.doom.weapons) do
			if not has then continue end
			if not dropWeapons[weapon] then continue end
			SpawnDropMobj(player.realmo, dropWeapons[weapon])
		end
	end
	
	if G_GametypeUsesLives() then
		player.lives = max($ - 1, 0)
	end
	
	local attacker = inflictor and inflictor.target or source
	if attacker and attacker.valid and attacker.player and attacker.player.valid then
		if not G_GametypeHasTeams() or ((attacker.ctfteam ~= target.ctfteam) and (attacker ~= target)) then
			P_AddPlayerScore(attacker.player, 100)
		end
	end
	
	target.flags = $ & ~(MF_NOGRAVITY|MF_NOCLIP|MF_NOCLIPHEIGHT)
	target.type = MT_DEADCORPSE
	
	if (player.doom.health > (100 * -FU)) then
		player.doom.gibbed = false
		target.state = S_PLAY_DOOMDEAD
		S_StartSound(target, (player.doom.health < (50 * -FU)) and sfx_plsuff or sfx_pldead, nil)
	else
		player.doom.gibbed = true
		target.state = S_PLAY_DOOMXDEAD
		S_StartSound(target, sfx_gibb, nil)
	end
	return true
end, MT_PLAYER)

addHook("MobjThinker", function(target)
	if not target.player or not target.player.valid then return end
	
	local player = target.player
	if not DG.PlayerIsDoomGuy(player) then return end
	if not player.doom then return end
	
	if player.doom.gibbed then
		if (target.state ~= S_PLAY_DOOMXDEAD) and (target.state ~= S_PLAY_DOOMXCORPSE) then
			target.state = S_PLAY_DOOMXDEAD
		end
	else
		if (target.state ~= S_PLAY_DOOMDEAD) and (target.state ~= S_PLAY_DOOMCORPSE) then
			target.state = S_PLAY_DOOMDEAD
		end
	end
end, MT_DEADCORPSE)