local DG = DoomGuy
local VSCALE = FU + (FU / 5)

local wpn_x, wpn_y = 0, 0
local wpn_xo, wpn_yo = 0, 0
local function drawWeapon(v, ply, cam, xoff, yoff, flags)
	if splitscreen then return end -- complicated due to some weapon sprites (see rocket launcher as an example)

	local bob = ply.bob
	local ww_xx = 0
	local ww_yy = 0
	local bob_calc = leveltime * (ANG2 + ANG2 + ANG2)
	if (ply.doom.state == WEAPONSTATE_READY) then
		ww_xx = FixedMul(sin(bob_calc), bob)
		ww_yy = abs(FixedMul(cos(bob_calc), bob))
	end
	
	if (ply.doom.state ~= WEAPONSTATE_ATTACK) then
		local diff_x = ww_xx - wpn_x
		local diff_y = ww_yy - wpn_y
		wpn_x = $ + diff_x
		wpn_y = $ + diff_y
		wpn_xo, wpn_yo = wpn_x, wpn_y
	end
	
	if not cam.chase then
		local colormap = nil
		local extra_flags = 0
		if ply.doom.powers[POWERS_INVISIBILITY] and ((ply.doom.powers[POWERS_INVISIBILITY] > (4 * 32)) or ((ply.doom.powers[POWERS_INVISIBILITY] & 8) and not doomclient.no_flashes.value)) then
			extra_flags = V_80TRANS
			colormap = v.getColormap(TC_RAINBOW, SKINCOLOR_BLACK)
		end
		
		for index = 0, #ply.doom.weapon_sprites, 1 do
			local weapon_sprite = ply.doom.weapon_sprites[index]
			if not weapon_sprite or not weapon_sprite.sprite then continue end
			
			local sprite = v.getSpritePatch(weapon_sprite.sprite, weapon_sprite.frame, 0, 0)
			v.drawStretched((ply.doom.weapon_xoffset * FU) + wpn_xo + xoff, (ply.doom.weapon_yoffset * FU) + wpn_yo + yoff, FU, VSCALE, sprite, flags|extra_flags, colormap)
		end
	end
end

local FACE_STATE_IDLE = 0
local FACE_STATE_BLINK = 1
local FACE_STATE_HURT = 2
local FACE_STATE_OUCH = 3
local FACE_STATE_EVIL = 4
local FACE_STATE_DEAD = 5
local FACE_STATE_GOD = 6

local face_weapons = {}
local face_random = 0
local face_priority = 0
local face_state = FACE_STATE_IDLE
local face_state_index = 1
local face_state_blood = 0
local face_tick = 0
local face_firing = 0
local face_prevhealth = 0
local face_table = {
	[FACE_STATE_IDLE] = {"STFIDLE", "STFSIDR", "STFSIDL"},
	[FACE_STATE_BLINK] = {"STFBLI"},
	[FACE_STATE_HURT] = {"STFHURT", "STFHURTR", "STFHURTL"},
	[FACE_STATE_OUCH] = {"STFOUCH"},
	[FACE_STATE_EVIL] = {"STFEVIL"},
	[FACE_STATE_DEAD] = {"STFDEAD"},
	[FACE_STATE_GOD] = {"STFGODIL", "STFGODVR", "STFGODVL", "STFGODBL", "STFGODHT", "STFGODEV", "STFGODOU"}
}

local function drawPlayerFace(v, ply, xpos, ypos, flags)
	face_random = v.RandomByte()
	
	local health = max(FixedInt(ply.doom.health), 0)
	local health_max = FixedInt(ply.doom.health_max)
	local player_health = (health > health_max) and health_max or health
	face_state_blood = ((health_max - player_health) * 5) / (health_max + 1)
	if (face_state == FACE_STATE_GOD) then
		face_state_blood = 0
	end
	
	if (face_priority < 10) then
		if (ply.playerstate == PST_DEAD) then
			face_state = FACE_STATE_DEAD
			face_state_index = 1
			face_priority = 9
			face_tick = 1
			face_state_blood = 0
		end
	end
	
	if (face_priority < 9) then
		for index, weapon in pairs(ply.doom.weapons) do
			if not face_weapons[index] or (face_weapons[index] ~= ply.doom.weapons[index]) then
				face_weapons[index] = ply.doom.weapons[index]
				face_state = FACE_STATE_EVIL
				face_state_index = 1
				face_priority = 8
				face_tick = 2 * TR
			end
		end
	end
	
	if (face_priority < 8) then
		if ply.doom.attacker_time and ply.doom.attacker and ply.doom.attacker.valid and (ply.doom.attacker ~= ply.realmo) then
			face_priority = 7
			if ((face_prevhealth - ply.doom.health) > (20 * FU)) then
				face_state = FACE_STATE_OUCH
				face_state_index = 1
				face_tick = TR
			elseif (face_state ~= FACE_STATE_OUCH) then
				local angle = R_PointToAngle2(ply.realmo.x, ply.realmo.y, ply.doom.attacker.x, ply.doom.attacker.y)
				local difference = 0
				local inverted = false
				if (angle > ply.realmo.angle)
					difference = (angle - ply.realmo.angle)
					inverted = (difference > ANGLE_180)
				else
					difference = (ply.realmo.angle - angle)
					inverted = (difference <= ANGLE_180)
				end
				
				face_state = FACE_STATE_HURT
				face_tick = TR
				
				face_state_index = inverted and 2 or 3
				if (difference < ANGLE_45) then
					face_state_index = 1
				end
			end
		end
	end

	if (face_priority < 7) then
		if ply.doom.hurted_flash then
			if ((face_prevhealth - ply.doom.health) > (20 * FU))
				face_priority = 7
				face_state = FACE_STATE_OUCH
				face_state_index = 1
			elseif (face_state ~= FACE_STATE_OUCH) then
				face_priority = 6
				face_state = FACE_STATE_HURT
				face_state_index = 1
			end
			face_tick = TR
		end
	end
	
	if (face_priority < 6) then
		if (ply.cmd.buttons & BT_ATTACK) then
			if (face_firing < (2 * TR))
				face_firing = $ + 1
			else
				face_priority = 5
				face_state = FACE_STATE_HURT
				face_state_index = 1
				face_tick = 1
			end
		else
			face_firing = 0
		end
	end
	
	if not face_tick then
		if (face_state ~= FACE_STATE_BLINK) and (face_state_index == 1) and v.RandomChance(FU / 5) then
			face_state = FACE_STATE_BLINK
			face_state_index = 1
			face_tick = TR / 8
		else
			face_state = FACE_STATE_IDLE
			face_state_index = (face_random % 3) + 1
			face_tick = TR / 2
		end
		face_priority = 0
	end
	face_tick = max($ - 1, 0)
	face_prevhealth = ply.doom.health
	
	local face_graphic = "STFIDLE0"
	local god_invencibility = ply.powers[pw_invulnerability]
	local god_flashing = ply.powers[pw_flashing]
	if (ply.pflags & PF_GODMODE) or ply.powers[pw_super] or god_invencibility or god_flashing then
		if (face_state == FACE_STATE_IDLE) then
			face_state_index = (face_random % 3) + 1
		elseif (face_state == FACE_STATE_BLINK) then
			face_state_index = 4
		elseif (face_state == FACE_STATE_HURT) then
			face_state_index = 5
		elseif (face_state == FACE_STATE_EVIL) then
			face_state_index = 6
		elseif (face_state == FACE_STATE_OUCH) then
			face_state_index = 7
		end
		face_state = FACE_STATE_GOD
		face_state_blood = 0
	end
	
	local extraflags = ply.spectator and V_TRANSLUCENT or 0
	face_graphic = face_table[face_state][face_state_index] .. face_state_blood
	v.drawStretched(xpos, ypos, FU, VSCALE, v.cachePatch(face_graphic), flags|extraflags, nil)
end

local function drawArmorIcon(v, ply, xpos, ypos, flags)
	local graphic = SPR_DABN
	if (ply.doom.armor_efficiency == (33 * FU)) then
		graphic = SPR_DAR0
	elseif (ply.doom.armor_efficiency == (50 * FU)) then
		graphic = SPR_DAR1
	end

	local sprite = v.getSpritePatch(graphic, A, 0, 0)
	v.drawStretched(xpos, ypos, FU, VSCALE, sprite, flags, nil)
end

local function drawFlashes(v, ply)
	if splitscreen then return end
	if doomclient.no_flashes.value then return end

	local color_flash = 0
	local color_flash_intensity = 0
	local damage_flash = ply.doom.hurted_flash
	if ply.doom.powers[POWERS_BERSERK] then
		local berserk_flash = 30 - (INT32_MAX - ply.doom.powers[POWERS_BERSERK]) >> 6
		if (berserk_flash > damage_flash)
			damage_flash = berserk_flash
		end
	end
	damage_flash = ($ + 7) >> 3

	local bonus_flash = (ply.doom.item_flash + 7) >> 3
	local hazardsuit_flash = 0
	if ply.doom.powers[POWERS_RADIATION_SUIT] and ((ply.doom.powers[POWERS_RADIATION_SUIT] > (4 * 32)) or (ply.doom.powers[POWERS_RADIATION_SUIT] & 8)) then
		hazardsuit_flash = 4
	end

	if damage_flash then
		color_flash = 37
		color_flash_intensity = min(damage_flash, 5)
	elseif bonus_flash then
		color_flash = 75
		color_flash_intensity = min(bonus_flash, 4)
	elseif hazardsuit_flash then
		color_flash = 104
		color_flash_intensity = hazardsuit_flash
	end
	
	if color_flash then
		v.fadeScreen(color_flash, max(min(color_flash_intensity, 10), 0))
	end
end

local function drawProgressBar(v, ply, x, y, amount, maximum, color, flags)
	local progress = min(FixedInt(FixedDiv(amount * FU, maximum * FU) * 30), 30)
	v.drawStretched(x * FU, y * FU, FU, VSCALE, v.cachePatch("STBARPB"), flags, nil)
	v.drawFill(x - (160 - 1), (y - FixedInt(FixedMul(35 * FU, VSCALE))) + 1, progress, 1, color|flags)
end

local function drawDefaultHUD(v, ply, cam)
	-- weapons and flashes
	drawWeapon(v, ply, cam, 0, -32 * FU, V_SNAPTOBOTTOM)
	drawFlashes(v, ply)

	-- status bar
	v.drawStretched(0, 200 * FU, FU, VSCALE, v.cachePatch("STBARRC"), V_SNAPTOBOTTOM|V_PERPLAYER, nil)
	v.drawStretched(320 * FU, 200 * FU, FU, VSCALE, v.cachePatch("STBARLC"), V_SNAPTOBOTTOM|V_PERPLAYER, nil)

	local statbar = v.cachePatch("STBAR")
	v.drawStretched(160 * FU, 200 * FU, FU, VSCALE, statbar, V_SNAPTOBOTTOM|V_PERPLAYER, nil)
	
	-- background color and face
	if netgame or multiplayer then
		v.drawStretched(160 * FU, (200 - 0) * FU, FU, VSCALE, v.cachePatch("STBARPL"), V_SNAPTOBOTTOM|V_PERPLAYER, v.getColormap(ply.realmo.skin, ply.skincolor))
	end
	drawPlayerFace(v, ply, 143 * FU, (163 * FU) - VSCALE, V_SNAPTOBOTTOM|V_PERPLAYER)
	
	-- running
	if ply.doom.sprinting then
		v.drawStretched((160 - 16) * FU, 194 * FU, FU, VSCALE, v.cachePatch("STBARRUN"), V_SNAPTOBOTTOM|V_PERPLAYER, v.getColormap(nil, SKINCOLOR_GOLD))
	end
	
	-- lives
	if G_GametypeUsesLives() and (CV_FindVar("cooplives").value > 0)
		drawCustomString((160 + 18) * FU, 194 * FU, ply.lives, FONT_DOOMTHIN, V_SNAPTOBOTTOM|V_PERPLAYER, {color = SKINCOLOR_GOLD, scale = FU, style = FS_FIXED, align = "right"})
	end
	
	-- health
	local health_color = SKINCOLOR_CLOUDY
	if (ply.doom.health < (ply.doom.health_max / 4)) then
		health_color = SKINCOLOR_RED
	elseif (ply.doom.health < (ply.doom.health_max / 2)) then
		health_color = SKINCOLOR_YELLOW
	elseif (ply.doom.health >= (ply.doom.health_max / 2)) then
		health_color = (ply.doom.health > ply.doom.health_max) and SKINCOLOR_BLUE or SKINCOLOR_GREEN
	end
	
	drawCustomString(115 * FU, 167 * FU, max(0, FixedInt(FixedCeil(ply.doom.health))) .. "%", FONT_DOOMNORMAL, V_SNAPTOBOTTOM|V_PERPLAYER, {color = SKINCOLOR_RED, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "center"})
	drawCustomString(115 * FU, 188 * FU, "HEALTH", FONT_DOOMCONSOLE, V_SNAPTOBOTTOM|V_PERPLAYER, {color = health_color, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "center"})
	
	-- armor
	local armor_color = SKINCOLOR_CLOUDY
	if (ply.doom.armor_efficiency == (33 * FU)) then
		armor_color = SKINCOLOR_GREEN
	elseif (ply.doom.armor_efficiency == (50 * FU)) then
		armor_color = SKINCOLOR_BLUE
	end
	drawCustomString(207 * FU, 167 * FU, max(0, FixedInt(FixedCeil(ply.doom.armor))) .. "%", FONT_DOOMNORMAL, V_SNAPTOBOTTOM|V_PERPLAYER, {color = SKINCOLOR_RED, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "center"})
	drawCustomString(207 * FU, 188 * FU, "ARMOR", FONT_DOOMCONSOLE, V_SNAPTOBOTTOM|V_PERPLAYER, {color = armor_color, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "center"})
	
	-- ammo (left)
	local wpn_ammo = "-"
	local wpn = DG.GetCurrentPlayerWeapon(ply)
	if wpn then
		if (wpn.ammo_type ~= nil) and (ply.doom.ammo[wpn.ammo_type] ~= nil) then
			wpn_ammo = ply.doom.ammo[wpn.ammo_type]
		end
	end
	drawCustomString(45 * FU, 167 * FU, wpn_ammo, FONT_DOOMNORMAL, V_SNAPTOBOTTOM|V_PERPLAYER, {color = SKINCOLOR_RED, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "center"})
	
	-- ammo (right)
	local ammos = {"clip", "shell", "rocket", "cell"}
	v.drawStretched(237 * FU, (200 - 37) * FU, FU, VSCALE, v.cachePatch("STBARAM"), V_SNAPTOBOTTOM|V_PERPLAYER, nil)
	
	for index = 1, #ammos, 1 do
		local offset = (index - 1) * 7
		drawCustomString((237 + 54) * FU, (200 - (31 - offset)) * FU, string.format("%03d", ply.doom.ammo[ammos[index]]), FONT_DOOMTHIN, V_SNAPTOBOTTOM|V_PERPLAYER, {color = SKINCOLOR_GOLD, scale = FU, style = FS_FIXED, align = "right"})
		drawCustomString((237 + 63) * FU, (200 - (31 - offset)) * FU, string.format("%03d", ply.doom.ammo_max[ammos[index]]), FONT_DOOMTHIN, V_SNAPTOBOTTOM|V_PERPLAYER, {color = SKINCOLOR_GOLD, scale = FU, style = FS_FIXED, align = "left"})
	end
	
	-- weapons
	v.drawStretched(4 * FU, (200 - 13) * FU, FU, VSCALE, v.cachePatch("STBARWP"), V_SNAPTOBOTTOM|V_PERPLAYER, nil)
	for index = 1, #DG.Weapons, 1 do
		local hasWeapon = false
		local hasAmmo = false
		local color = SKINCOLOR_GREY
		for slot, weapon in pairs(DG.Weapons[index]) do
			if not DG.PlayerHasWeapon(ply, slot, weapon) then continue end
			
			hasWeapon = true
			if DG.PlayerHasAmmoForWeapon(ply, weapon) then
				hasAmmo = true
				break
			end
		end
		
		if hasWeapon then
			color = hasAmmo and SKINCOLOR_GOLD or SKINCOLOR_CRIMSON
		end
		drawCustomString((10 + (11 * (index - 1))) * FU, 189 * FU, index, FONT_DOOMTHIN, V_SNAPTOBOTTOM|V_PERPLAYER, {color = color, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "center"})
	end
	
	-- oxygen bar
	if (ply.doom.oxygen < ply.doom.oxygen_max) then
		drawProgressBar(v, ply, 246, 200, ply.doom.oxygen / FU, ply.doom.oxygen_max / FU, 135, V_SNAPTOBOTTOM|V_PERPLAYER)
	end
	
	-- powerups bar
	if ply.doom.powers[POWERS_INVISIBILITY] then
		local fill = 0
		if ply.doom.powers[POWERS_INVISIBILITY] and ((ply.doom.powers[POWERS_INVISIBILITY] > (4 * 32)) or (ply.doom.powers[POWERS_INVISIBILITY] & 8)) then
			fill = ply.doom.powers[POWERS_INVISIBILITY] / TR
		end
		drawProgressBar(v, ply, 364 - 32, 200, fill, 30, 1, V_SNAPTOBOTTOM|V_PERPLAYER)
	end
	
	if doomclient.no_flashes.value and ply.doom.powers[POWERS_RADIATION_SUIT] then
		local fill = 0
		if ply.doom.powers[POWERS_RADIATION_SUIT] and ((ply.doom.powers[POWERS_RADIATION_SUIT] > (4 * 32)) or (ply.doom.powers[POWERS_RADIATION_SUIT] & 8)) then
			fill = ply.doom.powers[POWERS_RADIATION_SUIT] / TR
		end
		drawProgressBar(v, ply, 364, 200, fill, 60, 104, V_SNAPTOBOTTOM|V_PERPLAYER)
	end
end

local function drawFullscreenHUD(v, ply, cam)
	-- weapons and flashes
	drawWeapon(v, ply, cam, 0, 0, V_SNAPTOBOTTOM)
	drawFlashes(v, ply)
	
	-- face
	drawPlayerFace(v, ply, 2 * FU, (152 * FU) - VSCALE, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER)
	
	-- running
	if ply.doom.sprinting then
		v.drawStretched(2 * FU, 184 * FU, FU, VSCALE, v.cachePatch("STBARRUN"), V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER, v.getColormap(nil, SKINCOLOR_GOLD))
	end
	
	-- lives
	if G_GametypeUsesLives() and (CV_FindVar("cooplives").value > 0)
		drawCustomString(36 * FU, 184 * FU, ply.lives, FONT_DOOMTHIN, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER, {color = SKINCOLOR_GOLD, scale = FU, style = FS_FIXED, align = "right"})
	end
	
	-- health
	local health_color = SKINCOLOR_CLOUDY
	if (ply.doom.health < (ply.doom.health_max / 4)) then
		health_color = SKINCOLOR_RED
	elseif (ply.doom.health < (ply.doom.health_max / 2)) then
		health_color = SKINCOLOR_YELLOW
	elseif (ply.doom.health >= (ply.doom.health_max / 2)) then
		health_color = (ply.doom.health > ply.doom.health_max) and SKINCOLOR_BLUE or SKINCOLOR_GREEN
	end
	
	drawCustomString(44 * FU, 157 * FU, max(0, FixedInt(FixedCeil(ply.doom.health))) .. "%", FONT_DOOMNORMAL, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER, {color = SKINCOLOR_RED, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "left"})
	drawCustomString(44 * FU, 178 * FU, "HEALTH", FONT_DOOMCONSOLE, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER, {color = health_color, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "left"})
	
	-- armor icon
	drawArmorIcon(v, ply, 18 * FU, (148 * FU) - VSCALE, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER)
	
	-- armor
	local armor_color = SKINCOLOR_CLOUDY
	if (ply.doom.armor_efficiency == (33 * FU)) then
		armor_color = SKINCOLOR_GREEN
	elseif (ply.doom.armor_efficiency == (50 * FU)) then
		armor_color = SKINCOLOR_BLUE
	end
	drawCustomString(44 * FU, (157 - 38) * FU, max(0, FixedInt(FixedCeil(ply.doom.armor))) .. "%", FONT_DOOMNORMAL, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER, {color = SKINCOLOR_RED, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "left"})
	drawCustomString(44 * FU, (178 - 38) * FU, "ARMOR", FONT_DOOMCONSOLE, V_SNAPTOBOTTOM|V_SNAPTOLEFT|V_PERPLAYER, {color = armor_color, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "left"})
	
	-- current ammo
	local wpn_ammo = "-"
	local wpn = DG.GetCurrentPlayerWeapon(ply)
	if wpn then
		if (wpn.ammo_type ~= nil) and (ply.doom.ammo[wpn.ammo_type] ~= nil) then
			wpn_ammo = ply.doom.ammo[wpn.ammo_type]
		end
	end
	drawCustomString((320 - 84) * FU, 157 * FU, wpn_ammo, FONT_DOOMNORMAL, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER, {color = SKINCOLOR_RED, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "right"})
	drawCustomString((320 - 84) * FU, 178 * FU, "AMMO", FONT_DOOMCONSOLE, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER, {color = SKINCOLOR_CLOUDY, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "right"})
	
	-- weapons
	for index = 1, #DG.Weapons, 1 do
		local hasWeapon = false
		local hasAmmo = false
		local color = SKINCOLOR_GREY
		for slot, weapon in pairs(DG.Weapons[index]) do
			if not DG.PlayerHasWeapon(ply, slot, weapon) then continue end
			
			hasWeapon = true
			if DG.PlayerHasAmmoForWeapon(ply, weapon) then
				hasAmmo = true
				break
			end
		end
		
		if hasWeapon then
			color = hasAmmo and SKINCOLOR_GOLD or SKINCOLOR_CRIMSON
		end
		drawCustomString((243 + (11 * (index - 1))) * FU, 149 * FU, index, FONT_DOOMTHIN, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER, {color = color, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "center"})
	end
	
	-- ammos
	local ammos = {"clip", "shell", "rocket", "cell"}
	v.drawStretched(237 * FU, (190 - 37) * FU, FU, VSCALE, v.cachePatch("STBARAM"), V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER, nil)
	
	for index = 1, #ammos, 1 do
		local offset = (index - 1) * 7
		drawCustomString((237 + 54) * FU, (190 - (31 - offset)) * FU, string.format("%03d", ply.doom.ammo[ammos[index]]), FONT_DOOMTHIN, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER, {color = SKINCOLOR_GOLD, scale = FU, style = FS_FIXED, align = "right"})
		drawCustomString((237 + 63) * FU, (190 - (31 - offset)) * FU, string.format("%03d", ply.doom.ammo_max[ammos[index]]), FONT_DOOMTHIN, V_SNAPTOBOTTOM|V_SNAPTORIGHT|V_PERPLAYER, {color = SKINCOLOR_GOLD, scale = FU, style = FS_FIXED, align = "left"})
	end
	
	-- oxygen bar
	if (ply.doom.oxygen < ply.doom.oxygen_max) then
		local oxygen = FixedMul(FixedDiv(ply.doom.oxygen, ply.doom.oxygen_max), 100 * FU)
		drawCustomString(239 * FU, 138 * FU, "OXYGEN", FONT_PRBOOM, V_SNAPTORIGHT|V_SNAPTOBOTTOM|V_PERPLAYER, {color = SKINCOLOR_SKY, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "left"})
		drawCustomString((239 + 71) * FU, 138 * FU, FixedInt(oxygen) .. "%", FONT_PRBOOM, V_SNAPTORIGHT|V_SNAPTOBOTTOM|V_PERPLAYER, {color = SKINCOLOR_WHITE, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "right"})
	end
	
	-- powerups bar
	if doomclient.no_flashes.value and ply.doom.powers[POWERS_RADIATION_SUIT] then
		local color = ((ply.doom.powers[POWERS_RADIATION_SUIT] > (4 * 32)) or (ply.doom.powers[POWERS_RADIATION_SUIT] & 8)) and SKINCOLOR_GREEN or SKINCOLOR_RED
		drawCustomString(239 * FU, 128 * FU, "RADSUIT", FONT_PRBOOM, V_SNAPTORIGHT|V_SNAPTOBOTTOM|V_PERPLAYER, {color = color, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "left"})
		drawCustomString((239 + 71) * FU, 128 * FU, ply.doom.powers[POWERS_RADIATION_SUIT] / TR, FONT_PRBOOM, V_SNAPTORIGHT|V_SNAPTOBOTTOM|V_PERPLAYER, {color = SKINCOLOR_WHITE, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "right"})
	end
	
	if ply.doom.powers[POWERS_INVISIBILITY] then
		local color = ((ply.doom.powers[POWERS_INVISIBILITY] > (4 * 32)) or (ply.doom.powers[POWERS_INVISIBILITY] & 8)) and SKINCOLOR_GREY or SKINCOLOR_RED
		drawCustomString(239 * FU, 118 * FU, "INVIS", FONT_PRBOOM, V_SNAPTORIGHT|V_SNAPTOBOTTOM|V_PERPLAYER, {color = color, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "left"})
		drawCustomString((239 + 71) * FU, 118 * FU, ply.doom.powers[POWERS_INVISIBILITY] / TR, FONT_PRBOOM, V_SNAPTORIGHT|V_SNAPTOBOTTOM|V_PERPLAYER, {color = SKINCOLOR_WHITE, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "right"})
	end
end

hud.add(function(v, ply, cam)
	if not DG.PlayerIsDoomGuy(ply) then return end
	if not ply.realmo then return end
	if not ply.doom then return end
	if (ply.powers[pw_carry] == CR_NIGHTSMODE) then return end

	-- disable vanilla hud
	for index = 1, #DG.OverrideHUDs , 1 do
		if not hud.enabled(DG.OverrideHUDs[index]) continue end
		hud.disable(DG.OverrideHUDs[index])
	end
	
	-- determine type of hud
	if doomclient.fullscreen_hud.value then
		drawFullscreenHUD(v, ply, cam)
	else
		drawDefaultHUD(v, ply, cam)
	end
	
	-- weapon change
	if (((ply.doom.state ~= WEAPONSTATE_ATTACK) and (ply.doom.state ~= WEAPONSTATE_READY)) or splitscreen) and (ply.playerstate ~= PST_DEAD) and not ply.exiting and not ply.spectator then
		local weapon = DG.Weapons[ply.doom.pending_weapon_slot] and DG.Weapons[ply.doom.pending_weapon_slot][ply.doom.pending_weapon] or nil
		local weaponColor = (splitscreen and (ply.doom.state ~= WEAPONSTATE_ATTACK) and (ply.doom.state ~= WEAPONSTATE_READY)) and SKINCOLOR_CLOUDY or SKINCOLOR_YELLOW
		if weapon then
			local position = doomclient.fullscreen_hud.value and 16 or 56
			drawCustomString(160 * FU, (200 - position) * FU, weapon.name:upper(), FONT_DOOMCONSOLE, V_SNAPTOBOTTOM|V_20TRANS|V_PERPLAYER, {color = weaponColor, scale = FU, yscale = VSCALE, style = FS_FIXED|FS_FIXEDSTRECH, align = "center"})
		end
	end
end, "game")