local DG = DoomGuy
local RingWeaponToAmmo = {
	[MT_INFINITYRING] = "clip",
	[MT_AUTOMATICRING] = "clip",
	[MT_SCATTERRING] = "shell",
	[MT_BOUNCERING] = "shell",
	[MT_GRENADERING] = "rocket",
	[MT_EXPLOSIONRING] = "cell",
	[MT_RAILRING] = "cell"
}
local function RingWeaponAsAmmo(special, toucher)
	if not special or not special.valid or not toucher or not toucher.valid return end
	
	local player = toucher.player
	if not DG.PlayerIsDoomGuy(player) then return end
	if not player.doom then return end
	
	local ammo = RingWeaponToAmmo[special.type]
	if DG.GivePlayerAmmo(player, ammo, DG.Ammos[ammo].give_amount * 4) then
		player.doom.item_flash = $ + HUDFLASH_YELLOWADD
		S_StartSound(player.realmo, sfx_itempi, nil)
		P_KillMobj(special, nil, nil, 0)
	end
	return true
end
addHook("TouchSpecial", RingWeaponAsAmmo, MT_INFINITYRING)
addHook("TouchSpecial", RingWeaponAsAmmo, MT_AUTOMATICRING)
addHook("TouchSpecial", RingWeaponAsAmmo, MT_SCATTERRING)
addHook("TouchSpecial", RingWeaponAsAmmo, MT_BOUNCERING)
addHook("TouchSpecial", RingWeaponAsAmmo, MT_GRENADERING)
addHook("TouchSpecial", RingWeaponAsAmmo, MT_EXPLOSIONRING)
addHook("TouchSpecial", RingWeaponAsAmmo, MT_RAILRING)

local RingPanelToWeapon = {
	[MT_AUTOPICKUP] = {4, "chaingun"},
	[MT_SCATTERPICKUP] = {3, "shotgun"},
	[MT_BOUNCEPICKUP] = {3, "supershotgun"},
	[MT_GRENADEPICKUP] = {5, "rocketlauncher"},
	[MT_EXPLODEPICKUP] = {6, "plasmarifle"},
	[MT_RAILPICKUP] = {7, "bfg9000"}
}
local function RingPanelAsWeapon(special, toucher)
	if not special or not special.valid or not toucher or not toucher.valid return end
	
	local player = toucher.player
	if not DG.PlayerIsDoomGuy(player) then return end
	if not player.doom then return end
	
	local weapon = RingPanelToWeapon[special.type]
	local struct = DG.Weapons[weapon[1]][weapon[2]]
	if DG.GivePlayerWeapon(player, weapon[1], weapon[2]) then
		player.doom.item_flash = $ + HUDFLASH_YELLOWADD
		S_StartSound(player.realmo, sfx_wpnup, nil)
		P_KillMobj(special, nil, nil, 0)
	elseif DG.GivePlayerAmmo(player, struct.ammo_type, DG.Ammos[struct.ammo_type].give_amount * 2) then
		player.doom.item_flash = $ + HUDFLASH_YELLOWADD
		S_StartSound(player.realmo, sfx_wpnup, nil)
		P_KillMobj(special, nil, nil, 0)
	end
	return true
end
addHook("TouchSpecial", RingPanelAsWeapon, MT_AUTOPICKUP)
addHook("TouchSpecial", RingPanelAsWeapon, MT_SCATTERPICKUP)
addHook("TouchSpecial", RingPanelAsWeapon, MT_BOUNCEPICKUP)
addHook("TouchSpecial", RingPanelAsWeapon, MT_GRENADEPICKUP)
addHook("TouchSpecial", RingPanelAsWeapon, MT_EXPLODEPICKUP)
addHook("TouchSpecial", RingPanelAsWeapon, MT_RAILPICKUP)

addHook("TouchSpecial", function(special, toucher)
	if not special or not special.valid or not toucher or not toucher.valid return end
	
	local player = toucher.player
	if not DG.PlayerIsDoomGuy(player) then return end
	if not player.doom then return end
	if (special.state ~= S_EXTRALARGEBUBBLE) then return end
	if (toucher.eflags & MFE_VERTICALFLIP) then
		if (((special.z + special.height) < toucher.z) or ((special.z + special.height) > (toucher. z + (toucher.height * 2 / 3)))) then
			return
		end
	elseif ((special.z < toucher.z) or (special.z > (toucher.z + (toucher.height * 2 / 3))))
		return
	end
	
	player.doom.oxygen = player.doom.oxygen_max
	player.doom.item_flash = $ + HUDFLASH_YELLOWADD
	S_StartSound(player.realmo, sfx_itempi, nil)
end, MT_EXTRALARGEBUBBLE)

addHook("PlayerHeight", function(player)
	if not DG.PlayerIsDoomGuy(player) then return end
	if not player.doom then return end
	return (player.doom.crouching or ((player.mo.ceilingz - player.mo.floorz) < player.height)) and player.spinheight or player.height
end)

addHook("PlayerCanEnterSpinGaps", function(player)
	if not DG.PlayerIsDoomGuy(player) then return end
	if not player.doom then return end
	return true
end)