local DG = DoomGuy

local WEAPONTOP = 0
local WEAPONBOTTOM = 128
local RAISESPEED = 6
local LOWERSPEED = 6
DG.ExecutePlayerWeaponState = function(player, weapon)
	if weapon.states[player.doom.state][player.doom.state_index].func then
		weapon.states[player.doom.state][player.doom.state_index].func(player, weapon, unpack(weapon.states[player.doom.state][player.doom.state_index].param))
	end
end

DG.UpdatePlayerWeaponSprites = function(player, weapon)
	if weapon and weapon.states[player.doom.state] then
		if (type(weapon.states[player.doom.state][player.doom.state_index].sprite) == "function") then
			player.doom.weapon_sprites[WEAPONSPRITE_WEAPON].sprite, player.doom.weapon_sprites[WEAPONSPRITE_WEAPON].frame = weapon.states[player.doom.state][player.doom.state_index].sprite(player, weapon)
		else
			player.doom.weapon_sprites[WEAPONSPRITE_WEAPON].sprite = weapon.states[player.doom.state][player.doom.state_index].sprite
			player.doom.weapon_sprites[WEAPONSPRITE_WEAPON].frame = weapon.states[player.doom.state][player.doom.state_index].frame
		end
		
		if (type(weapon.states[player.doom.state][player.doom.state_index].fsprite) == "function") then
			player.doom.weapon_sprites[WEAPONSPRITE_FLASH].sprite, player.doom.weapon_sprites[WEAPONSPRITE_FLASH].frame = weapon.states[player.doom.state][player.doom.state_index].fsprite(player, weapon)
		else
			player.doom.weapon_sprites[WEAPONSPRITE_FLASH].sprite = weapon.states[player.doom.state][player.doom.state_index].fsprite
			player.doom.weapon_sprites[WEAPONSPRITE_FLASH].frame = weapon.states[player.doom.state][player.doom.state_index].fframe
		end
	end
end

addHook("PreThinkFrame", do
	for player in players.iterate do
		if not DG.PlayerIsDoomGuy(player) then continue end
		if not player.doom then continue end
		if (player.playerstate == PST_DEAD) then continue end
		if (player.powers[pw_carry] == CR_NIGHTSMODE) then continue end

		if (player.rings > player.doom.rings) then
			while (player.rings > player.doom.rings) do
				if (player.doom.health < (player.doom.health_max * 2)) then
					player.doom.health = min($ + FU, player.doom.health_max * 2)
					if (player.doom.armor < (player.doom.armor_max * 2)) and P_RandomChance(FU / 32) then
						if not player.doom.armor_efficiency then
							player.doom.armor_efficiency = 33 * FU
						end
						player.doom.armor = min($ + FU, player.doom.armor_max * 2)
					end
					S_StartSound(player.realmo, sfx_itempi, nil)
				end
				player.doom.rings = $ + 1
			end
		elseif (player.rings < player.doom.rings)
			player.doom.rings = player.rings
		end

		if player.powers[pw_shield] then
			if not player.doom.armor_efficiency then
				player.doom.armor_efficiency = 33 * FU
			end
			
			if (player.doom.armor < (player.doom.armor_max * 2)) then
				local amount = 15
				if (player.powers[pw_shield] == SH_PINK) then
					amount = 5
				elseif (player.powers[pw_shield] == SH_PITY) then
					amount = 10
				elseif (player.powers[pw_shield] == SH_WHIRLWIND) then
					amount = 20
				elseif (player.powers[pw_shield] == SH_ARMAGEDDON) then
					if (player.doom.armor_efficiency < (50 * FU)) then
						player.doom.armor_efficiency = 50 * FU
					end
					amount = 50
				end
				
				if (player.powers[pw_shield] & SH_FORCE) then
					if (player.doom.armor_efficiency < (50 * FU)) then
						player.doom.armor_efficiency = 50 * FU
					end
					amount = $ + (30 * ((player.powers[pw_shield] & SH_FORCEHP) + 1))
				end
				
				if (player.powers[pw_shield] & SH_FIREFLOWER) then
					amount = $ + 25
				end
				player.doom.armor = min($ + (amount * FU), player.doom.armor_max * 2)
			end
			player.doom.item_flash = $ + HUDFLASH_YELLOWADD
		end
		player.powers[pw_shield] = 0

		player.powers[pw_underwater] = 30 * TICRATE
		player.powers[pw_spacetime] = (11 * TICRATE) + (TICRATE / 2)
		player.powers[pw_noautobrake] = 2
		player.powers[pw_automaticring] = 0
		player.powers[pw_infinityring] = 0
		player.powers[pw_scatterring] = 0
		player.powers[pw_bouncering] = 0
		player.powers[pw_grenadering] = 0
		player.powers[pw_railring] = 0
		
		player.pflags = $ | (PF_THOKKED|PF_NOJUMPDAMAGE)
		player.pflags = $ & ~(PF_SPINNING)

		player.weapondelay = 69
		player.ringweapons = 0
		
		player.doom.tick_speed = 1
		if player.powers[pw_super] or player.powers[pw_sneakers] then
			player.doom.tick_speed = 2
		end
		
		player.doom.attacker_time = max($ - 1, 0)
		player.doom.hurted_flash = max($ - 1, 0)
		player.doom.item_flash = max($ - 1, 0)
		player.doom.invulnerability_tick = max($ - 1, 0)
	end
end)

local function HandlePowerups(player)
	if player.doom.powers[POWERS_BERSERK] then
		player.doom.powers[POWERS_BERSERK] = max($ - 1, 0)
	end
	
	if player.doom.powers[POWERS_RADIATION_SUIT] then
		player.doom.powers[POWERS_RADIATION_SUIT] = max($ - 1, 0)
	end
	
	if player.doom.powers[POWERS_INVISIBILITY] then
		player.doom.powers[POWERS_INVISIBILITY] = max($ - 1, 0)
	end
end

local function HandleWeaponChangeByInput(player)
	if (player.cmd.buttons & BT_WEAPONNEXT) and not (player.doom.previous_buttons & BT_WEAPONNEXT) then
		local avaliable = {}
		for index, slot in pairs(DG.Weapons) do
			for name, weapon in pairs(slot) do
				if not avaliable[index] then
					avaliable[index] = {}
				end
				if not DG.PlayerCanUseWeapon(player, name, weapon) then continue end
				table.insert(avaliable[index], name)
			end
		end
		
		local slot = player.doom.pending_weapon_slot
		local index = player.doom.weapon_slot_index
		for tries = 1, 16, 1 do
			if not avaliable[slot] then
				slot = $ + 1
				if (slot > #avaliable) then
					slot = 1
				end
				continue
			end
		
			if (#avaliable[slot] > 1) then
				index = $ + 1
				if (index > #avaliable[slot]) then
					slot = $ + 1
					if (slot > #avaliable) then
						slot = 1
					end
					index = 1
				end
			else
				slot = $ + 1
				if (slot > #avaliable) then
					slot = 1
				end
				index = 1
			end
			
			if avaliable[slot][index] then
				local selected = avaliable[slot][index]
				player.doom.pending_weapon = selected
				player.doom.pending_weapon_slot = slot
				player.doom.weapon_slot_index = index
				break
			end
		end
	end

	if (player.cmd.buttons & BT_WEAPONPREV) and not (player.doom.previous_buttons & BT_WEAPONPREV) then
		local avaliable = {}
		for index, slot in pairs(DG.Weapons) do
			for name, weapon in pairs(slot) do
				if not avaliable[index] then
					avaliable[index] = {}
				end
				if not DG.PlayerCanUseWeapon(player, name, weapon) then continue end
				table.insert(avaliable[index], name)
			end
		end
		
		local slot = player.doom.pending_weapon_slot
		local index = player.doom.weapon_slot_index
		for tries = 1, 16, 1 do
			if not avaliable[slot] then
				slot = $ - 1
				if (slot < 1) then
					slot = #avaliable
				end
				continue
			end
		
			if (#avaliable[slot] > 1) then
				index = $ - 1
				if (index < 1) then
					slot = $ - 1
					if (slot < 1) then
						slot = #avaliable
					end
					index = #avaliable[slot]
				end
			else
				slot = $ - 1
				if (slot < 1) then
					slot = #avaliable
				end
				index = #avaliable[slot]
			end
			
			if avaliable[slot][index] then
				local selected = avaliable[slot][index]
				player.doom.pending_weapon = selected
				player.doom.pending_weapon_slot = slot
				player.doom.weapon_slot_index = index
				break
			end
		end
	end

	if (player.cmd.buttons & BT_WEAPONMASK) and not (player.doom.previous_buttons & BT_WEAPONMASK) then
		local slot = (player.cmd.buttons & BT_WEAPONMASK)
		if DG.Weapons[slot] then
			local avaliable = {}
			for name, weapon in pairs(DG.Weapons[slot]) do
				if not DG.PlayerCanUseWeapon(player, name, weapon) then continue end
				table.insert(avaliable, name)
			end
			
			if (#avaliable > 0) then
				for index = 1, #avaliable, 1 do
					if (player.doom.pending_weapon_slot == slot) then
						player.doom.weapon_slot_index = $ + 1
						if (player.doom.weapon_slot_index > #avaliable) then
							player.doom.weapon_slot_index = 1
						end
					else
						player.doom.weapon_slot_index = 1
					end
					
					local selected = avaliable[player.doom.weapon_slot_index]
					player.doom.pending_weapon = selected
					player.doom.pending_weapon_slot = slot
					break
				end
			end
		end
	end
end

local function CheckWeaponChange(player)
	return (player.doom.weapon ~= player.doom.pending_weapon) or (player.doom.weapon_slot ~= player.doom.pending_weapon_slot)
end

local function DoWeaponChange(player)
	player.doom.state = WEAPONSTATE_LOWER
	player.doom.state_index = 1
	if weapon and weapon.states[player.doom.state] then
		player.doom.weapon_sprites[WEAPONSPRITE_WEAPON].sprite = weapon.states[player.doom.state][player.doom.state_index].sprite
		player.doom.weapon_sprites[WEAPONSPRITE_WEAPON].frame = weapon.states[player.doom.state][player.doom.state_index].frame
		player.doom.tick = weapon.states[player.doom.state][player.doom.state_index].tics
		DG.ExecutePlayerWeaponState(player, weapon)
	end
end

local function HandleWeaponStates(player)
	if (player.playerstate == PST_DEAD) or player.exiting or player.spectator then
		player.doom.state = WEAPONSTATE_LOWER
		player.doom.state_index = 1
		player.doom.tick = 0
	end

	local weapon = DG.GetCurrentPlayerWeapon(player)
	local tick_speed = player.doom.tick_speed
	if weapon and weapon.states[player.doom.state] and (weapon.states[player.doom.state][player.doom.state_index].flags & WEAPONSTATEFLAG_NOSPEEDMODIFIER) then
		tick_speed = 1
	end
	
	if CheckWeaponChange(player) and (player.doom.state == WEAPONSTATE_READY) then
		DoWeaponChange(player)
	end

	if (player.doom.state == WEAPONSTATE_CHANGE) then
		player.doom.weapon = player.doom.pending_weapon
		player.doom.weapon_slot = player.doom.pending_weapon_slot
		player.doom.state = WEAPONSTATE_RAISE
		player.doom.state_index = 1
		
		local weapon = DG.GetCurrentPlayerWeapon(player)
		if weapon and weapon.states[player.doom.state] then
			player.doom.tick = weapon.states[player.doom.state][player.doom.state_index].tics
			DG.ExecutePlayerWeaponState(player, weapon)
		end
	elseif (player.doom.state == WEAPONSTATE_RAISE) then
		player.doom.weapon_yoffset = max(WEAPONTOP, $ - RAISESPEED)
		if (player.doom.weapon_yoffset <= WEAPONTOP) then
			if CheckWeaponChange(player) then
				DoWeaponChange(player)
			else
				player.doom.state = WEAPONSTATE_READY
				player.doom.state_index = 1
				if weapon and weapon.states[player.doom.state] then
					player.doom.tick = weapon.states[player.doom.state][player.doom.state_index].tics
					DG.ExecutePlayerWeaponState(player, weapon)
				end
			end
		end
	elseif (player.doom.state == WEAPONSTATE_LOWER) then
		player.doom.weapon_yoffset = min(WEAPONBOTTOM, $ + LOWERSPEED)
		if (player.doom.weapon_yoffset >= WEAPONBOTTOM) then
			player.doom.state = WEAPONSTATE_CHANGE
			player.doom.state_index = 1
			if weapon and weapon.states[player.doom.state] then
				player.doom.tick = weapon.states[player.doom.state][player.doom.state_index].tics
				DG.ExecutePlayerWeaponState(player, weapon)
			end
		end
	elseif weapon then
		if (player.doom.state == WEAPONSTATE_ATTACK) then
			if (player.realmo.state ~= S_PLAY_DOOMFIRING) and (player.realmo.state ~= S_PLAY_DOOMFIRING_SHOOT) then
				player.realmo.state = S_PLAY_DOOMFIRING
			end
		elseif (player.realmo.state == S_PLAY_DOOMFIRING) or (player.realmo.state == S_PLAY_DOOMFIRING_SHOOT) then
			player.realmo.tics = 1
		end
	
		player.doom.tick = max($ - tick_speed, 0)
		if not player.doom.tick then
			player.doom.state_index = $ + 1
			if (player.doom.state_index > #weapon.states[player.doom.state]) then
				player.doom.state = WEAPONSTATE_READY
				player.doom.state_index = 1
				player.doom.tick = weapon.states[player.doom.state][player.doom.state_index].tics
				DG.ExecutePlayerWeaponState(player, weapon)
			else
				player.doom.tick = weapon.states[player.doom.state][player.doom.state_index].tics
				DG.ExecutePlayerWeaponState(player, weapon)
			end
		end
		
		if (player.doom.state == WEAPONSTATE_READY) then
			if (player.cmd.buttons & BT_ATTACK) then
				if DG.PlayerCanUseWeapon(player, player.doom.weapon, weapon) then
					if not player.doom.firing or (weapon.flags & WEAPONFLAG_AUTOFIRE) then
						player.doom.state = WEAPONSTATE_ATTACK
						player.doom.state_index = 1
						player.doom.tick = weapon.states[player.doom.state][player.doom.state_index].tics
						DG.ExecutePlayerWeaponState(player, weapon)
						
						player.doom.firing = $ + 1
						player.realmo.state = S_PLAY_DOOMFIRING
					end
				end
			else
				player.doom.firing = 0
			end
		end
	end
	DG.UpdatePlayerWeaponSprites(player, weapon)
end

local function HandleMisc(player)
	if (player.playerstate ~= PST_DEAD) then
		if ((player.realmo.eflags & MFE_UNDERWATER) or P_InSpaceSector(player.realmo)) and not ((player.pflags & PF_GODMODE) or player.powers[pw_super] or player.powers[pw_invulnerability]) then
			player.doom.oxygen = max($ - player.doom.oxygen_drainrate, 0)
			if not player.doom.oxygen and not (leveltime & 31) and not (player.exiting or (player.pflags & PF_FINISHED)) then
				local damagetype = P_InSpaceSector(player.realmo) and DAMAGETYPE_SPACEDROWING or DAMAGETYPE_DROWING
				P_DamageMobj(player.realmo, nil, nil, 10, damagetype)
			end
		elseif (player.doom.oxygen < player.doom.oxygen_max) then
			player.doom.oxygen = min($ + player.doom.oxygen_gainrate, player.doom.oxygen_max)
		end
		
		if (player.realmo.eflags & MFE_UNDERWATER) then
			local force = player.doom.sprinting and (FU / 2) or (FU / 4)
			local limit_momz = (abs(player.realmo.momz) <= ((10 * player.realmo.scale) * P_MobjFlip(player.realmo)))
			if (player.cmd.buttons & (BT_JUMP|BT_SPIN)) and not player.powers[pw_nocontrol] and limit_momz then
				P_SetObjectMomZ(player.realmo, (player.cmd.buttons & BT_SPIN) and -force or force, true)
			end
		end
		player.doom.crouching = ((player.cmd.buttons & BT_SPIN) and (player.powers[pw_carry] == CR_NONE))
		
		if (player.cmd.buttons & BT_FIRENORMAL) and not (player.doom.previous_buttons & BT_FIRENORMAL) then
			player.doom.sprinting = not player.doom.sprinting
		end
		
		if (player.cmd.buttons & BT_TOSSFLAG) and not (player.doom.previous_buttons & BT_TOSSFLAG) then
			S_StartSound(player.realmo, sfx_ploof, nil)
		end
		
		if (player.realmo.eflags & MFE_JUSTHITFLOOR) and not P_CheckDeathPitCollide(player.realmo) then
			if (FixedInt(player.doom.previous_momz * (P_MobjFlip(player.realmo) * -1)) > 12) then
				S_StartSound(player.realmo, sfx_ploof, nil)
			end
		end
		
		if player.doom.hurted_flash then
			player.realmo.state = S_PLAY_PAIN
		end
		player.doom.previous_momz = player.realmo.momz
	end
	player.realmo.spriteyscale = (FU / ((player.realmo.height <= player.spinheight) and 2 or 1)) + (FU / 5)
	
	local speed_modifier = player.doom.sprinting and 2 or 1
	player.normalspeed = skins[player.skin].normalspeed * speed_modifier
	player.doom.previous_buttons = player.cmd.buttons
end

addHook("ThinkFrame", do
	for player in players.iterate do
		if not DG.PlayerIsDoomGuy(player) then
			if player.doom then
				if (player == consoleplayer) then
					for index = 1, #DG.OverrideHUDs , 1 do
						hud.enable(DG.OverrideHUDs[index])
					end
				end
				player.doom = nil
				player.realmo.spriteyscale = FU
			end
			continue
		end
		if (player.powers[pw_carry] == CR_NIGHTSMODE) then continue end
		
		if not player.doom then
			DG.SetupPlayer(player)
			
			if not player.spectator then
				local effect = P_SpawnMobjFromMobj(player.realmo, 0, 0, 0, MT_DTELEPORT)
				effect.spriteyscale = FU + (FU / 5)
				S_StartSound(player.realmo, sfx_telepo)
			end
			continue
		end
		HandlePowerups(player)
		HandleWeaponChangeByInput(player)
		HandleWeaponStates(player)
		HandleMisc(player)
	end
end)

addHook("PostThinkFrame", do
	for player in players.iterate do
		if not DG.PlayerIsDoomGuy(player) then continue end
		if not player.doom then continue end
		
		if (player.doom.state == WEAPONSTATE_ATTACK) then
			player.drawangle = player.realmo.angle
		end
		
		if player.doom.powers[POWERS_INVISIBILITY] and ((player.doom.powers[POWERS_INVISIBILITY] > (4 * 32)) or (player.doom.powers[POWERS_INVISIBILITY] & 8)) then
			player.realmo.flags2 = $ | MF2_SHADOW
			player.realmo.color = SKINCOLOR_BLACK
			player.realmo.colorized = true
		elseif player.realmo.colorized then
			player.realmo.color = player.skincolor
			player.realmo.colorized = false
		end
		
		if (player.playerstate == PST_DEAD) then
			player.aiming = FixedDiv($, FU + (FU / 5))
			
			local target = player.doom.attacker
			if target and target.valid then
				local angle = R_PointToAngle2(player.realmo.x, player.realmo.y, target.x, target.y)
				local difference = FixedInt(AngleFixed(angle - player.realmo.angle))
				if (difference < 5) or (difference > 355) then
					player.realmo.angle = angle
				elseif (difference < 180) then
					player.realmo.angle = $ + (ANG10 / 2)
				else
					player.realmo.angle = $ - (ANG10 / 2)
				end
			end
		end
		
		local viewheight = (player.playerstate ~= PST_DEAD) and FixedMul(41 * player.realmo.height / 48, player.realmo.scale) or 0
		player.deltaviewheight = (viewheight - player.viewheight) / 4
	end
end)